// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO16/16AO16_Linux_2.x.x.x_DN/samples/sbtest/channel_sel.c $
// $Rev: 53282 $
// $Date: 2023-06-28 14:15:26 -0500 (Wed, 28 Jun 2023) $

// 16AO16: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	service_data_t	list[]	=
	{
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AO16_IOCTL_CHANNEL_SEL,
			/* arg		*/	-1,
			/* reg		*/	AO16_GSC_CSR,
			/* mask		*/	-1,
			/* value	*/	-1
		},
		{
			/* service	*/	SERVICE_NORMAL,
			/* cmd		*/	AO16_IOCTL_CHANNEL_SEL,
			/* arg		*/	-1,
			/* reg		*/	AO16_GSC_CSR,
			/* mask		*/	-1,
			/* value	*/	-1
		},

		{ SERVICE_END_LIST }
	};

	s32	chans;
	int	errs	= 0;
	int	i;
	u32	mask;
	u32	value;

	errs	+= ao16_query(fd, -1, 0, AO16_QUERY_CHANNEL_QTY, &chans);

	if (errs)
	{
	}
	else if ((chans != 8) && (chans != 12) && (chans != 16))
	{
		errs++;
		printf(	"FAIL <---  (channel quantity query error: %ld)\n",
				(long) chans);
	}
	else
	{
		errs			+= ao16_initialize(fd, -1, 0);
		mask			= 0xFFFFFFFF >> (32 - chans);
		list[0].mask	= mask;
		list[1].mask	= mask;

		for (i = 0; i < chans; i++)
		{
			value			= 0x1L << i;
			list[0].arg		= value;
			list[0].value	= value;
			list[1].arg		= mask ^ value;
			list[1].value	= mask ^ value;

			errs	+= service_ioctl_set_reg_list(fd, list);
			errs	+= service_ioctl_reg_get_list(fd, list);

			errs	+= service_ioctl_set_reg_list(fd, list);
			errs	+= service_ioctl_reg_get_list(fd, list);
		}

		errs	+= ao16_initialize(fd, -1, 0);
	}

	return(errs);
}



//*****************************************************************************
static int _function_test_option(int fd, int fsamp, int chans)
{
	#define	SAMPLES	600	// 1 chan = 3 secs, 2 chans = 1.5 secs

	s32	empty;
	s32	empty_not;
	u32	buf[SAMPLES];
	int	delta	= 500;	// 500 ms window before and after empty mark
	int	errs	= 0;
	int	mask	= GSC_FIELD_ENCODE(0xFFFF, chans - 1, 0);
	int	ms;
	int	send	= (int) sizeof(buf);
	int	sent;

	for (;;)	// A convenience loop.
	{
		ms	= 1000 * SAMPLES / chans / fsamp;

		// We check the FIFO status before and after the empty mark
		memset(buf, 0, sizeof(buf));
		errs	+= ao16_config_ao	(fd, -1, 0, fsamp);
		errs	+= ao16_channel_sel	(fd, -1, 0, mask, NULL);
		errs	+= ao16_clock_enable(fd, -1, 0, AO16_CLOCK_ENABLE_NO, NULL);
		errs	+= ao16_buffer_clear(fd, -1, 0);
		sent	= ao16_write(fd, buf, send);
		errs	+= ao16_clock_enable(fd, -1, 0, AO16_CLOCK_ENABLE_YES, NULL);
		gsc_time_sleep_ms(ms - delta);
		errs	+= ao16_buffer_status(fd, -1, 0, &empty_not);
		gsc_time_sleep_ms(delta * 2);
		errs	+= ao16_buffer_status(fd, -1, 0, &empty);
		errs	+= ao16_initialize(fd, -1, 0);

		if (errs)
		{
			printf("FAIL <---  (Service error.)\n");
			break;
		}

		if (sent != send)
		{
			errs++;
			printf(	"ERROR: %d. write: sent %ld bytes, expected %ld)\n",
					__LINE__,
					(long) sent,
					(long) send);
			break;
		}

		if (empty_not == AO16_BUFFER_STATUS_EMPTY)
		{
			errs++;
			printf("FAIL <--- (Chans: %d, FIFO should not be empty)\n", chans);
		}
		else if (empty != AO16_BUFFER_STATUS_EMPTY)
		{
			errs++;
			printf("FAIL <--- (Chans: %d, FIFO not EMPTY.)\n", chans);
		}

		break;
	}

	return(errs);
}



//*****************************************************************************
static int _function_test(int fd)
{
	// The procedure is to select a sample rate and data volume so we can use
	// the time it takes for the buffer to empty. This is done with just one
	// channel enabled and with two enabled. The time needed to empty the
	// buffer is then 1X and 2X, making it easy to test this feature. This test
	// was modified to support the plain 16AO16 and the 16AO16FLV, which have
	// different reference clock frequencies.

	#define	FSAMP	200	// very low rate supported by all devices

	int	errs	= 0;

	errs	+= _function_test_option(fd, FSAMP, 1);
	errs	+= _function_test_option(fd, FSAMP, 2);

	return(errs);
}



/******************************************************************************
*
*	Function:	channel_sel_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service AO16_IOCTL_CHANNEL_SEL.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int channel_sel_test(int fd)
{
	int	errs	= 0;

	gsc_label("AO16_IOCTL_CHANNEL_SEL");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}



