// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO16/16AO16_Linux_2.x.x.x_DN/utils/util_buffer_status.c $
// $Rev: 52413 $
// $Date: 2023-01-19 15:50:37 -0600 (Thu, 19 Jan 2023) $

// 16AO16: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao16_buffer_status
*
*	Purpose:
*
*		Provide a visual wrapper for the AO16_IOCTL_BUFFER_STATUS service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		get		The current setting is recorded here, if not NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao16_buffer_status(int fd, int index, int verbose, s32* get)
{
	char	buf[128];
	int		errs;
	int		ret;
	s32		set		= -1;

	if (verbose)
		gsc_label_index("Buffer Status", index);

	ret		= ao16_ioctl(fd, AO16_IOCTL_BUFFER_STATUS, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			sprintf(buf, "INVALID: %ld", (long) get[0]);
			break;

		case AO16_BUFFER_STATUS_EMPTY:

			strcpy(buf, "Empty");
			break;

		case AO16_BUFFER_STATUS_1Q_FULL:

			strcpy(buf, "< 1/4 Full");
			break;

		case AO16_BUFFER_STATUS_MEDIUM:

			strcpy(buf, "Medium");
			break;

		case AO16_BUFFER_STATUS_3Q_FULL:

			strcpy(buf, ">= 3/4 Full");
			break;

		case AO16_BUFFER_STATUS_FULL:

			strcpy(buf, "Full");
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", buf);

	if (get)
		get[0]	= set;

	return(errs);
}


